@@ -1,5 +1,7 @@
 package Win32::Daemon::Simple;
+use POSIX qw(strftime);
 use Win32;
+use Hash::Case::Preserve;
 use Win32::Console qw();
 use Win32::Daemon;
 use FindBin qw($Bin $Script);$Bin =~ s{/}{\\}g;
@@ -11,19 +13,28 @@ use strict;
 use vars qw(@ISA @EXPORT $VERSION);
 @ISA = qw(Exporter);
 @EXPORT = qw(ReadParam SaveParam Log LogNT OpenLog CloseLog CatchMessages GetMessages ServiceLoop DoEvents CMDLINE);
-$VERSION = '0.1.3';
+$VERSION = '0.2.2';
+
+sub Now () {
+	strftime( "%Y/%m/%d %H:%M:%S", localtime())
+}
+
+my $compiled = !(-e $INC{'Win32/Daemon/Simple.pm'}); # if the module was not read from disk => the script has been "compiled"
+if (0) {
+	require 'Win32/Service.pm';
+}
 
 my ($svcid, $svcname, $svcversion) = ( $Script, $Script);
 BEGIN {
 	eval {
 		my $title;
-		if ($title = Win32::Console::_GetConsoleTitle()) {
+		if ($^C != 1 and $title = Win32::Console::_GetConsoleTitle()) {
 			eval 'sub CMDLINE () {1}';
 			if (lc($title) eq lc($^X) or lc($title) eq lc($0)) {
 				eval 'sub FROMCMDLINE () {0}';
 				eval '*MsgBox = \&Win32::MsgBox;';
 				if (! @ARGV) {
-					push @ARGV, '-help';
+					push @ARGV, '-help_and_settings';
 				}
 			} else {
 				eval 'sub FROMCMDLINE () {1}';
@@ -37,7 +48,7 @@ BEGIN {
 	};
 }
 
-my ($info, $params, $param_modify);
+my ($info, $params, $param_modify, $run_params);
 
 # parameters
 use Win32::Registry;
@@ -60,10 +71,11 @@ use Win32::Registry;
 		my ( $param, $value) = @_;
 		$key = $HKLM->Open('SYSTEM\CurrentControlSet\Services\\'.$svcid)
 			unless $key;
+		return unless $key; # do not save anything if the service is not installed
 		$paramkey = $key->Create('Parameters')
 			unless $paramkey;
+		die "Can't open/create the Parameters subkey in the service settings!\n" unless $paramkey; # do not save anything if the service is not installed
 		if (!defined $value) {
-print "Delete param: $param\n";
 			$paramkey->DeleteValue($param);
 		} elsif ($value =~ /^\d+$/) {
 			$paramkey->SetValues($param, REG_DWORD, $value);
@@ -74,9 +86,59 @@ print "Delete param: $param\n";
 	}
 }
 
+sub StopStartService {
+	$|=1;
+	my $cmd = shift;
+	eval "use Win32::Service";
+	my %status;
+	Win32::Service::GetStatus(undef, $svcid, \%status);
+	if (! %status) {
+		print "\t\tThe service $svcid doesn't exist!\n";
+		next;
+	};
+	my ($good_status,$msg);
+	if ($cmd eq 'stop') {
+		$good_status = 1;
+		if( $status{CurrentState} == $good_status) {
+			print "\n    The service did not run.\n";
+			MsgBox "The service did not run.\n", MB_ICONINFORMATION, $svcname;
+			return;
+		}
+		$msg = "stopped";
+		Win32::Service::StopService(undef, $svcid);sleep(1);
+	} else {
+		$good_status = 4;
+		if( $status{CurrentState} == $good_status) {
+			print "\n    The service was running already.\n";
+			MsgBox "The service was running already.\n", MB_ICONINFORMATION, $svcname;
+			return;
+		}
+		$msg = "started";
+		Win32::Service::StartService(undef, $svcid);sleep(1);
+	}
+
+	my $count=0;
+	while (++$count < 120 and $status{CurrentState} != $good_status) {
+		sleep(1);
+		print '.';
+		Win32::Service::GetStatus(undef, $svcid, \%status);
+	}
+
+	if( $status{CurrentState} == $good_status) {
+		print "   The service was $msg.\n";
+		MsgBox "The service was $msg.\n", MB_ICONINFORMATION, $svcname;
+	} else {
+		print "   The service could not be $msg.\n";
+		MsgBox "The service could not be $msg.\n", MB_ICONERROR, $svcname;
+	}
+}
+
 my ($logging_code,$loop_code);
 # main processing
 sub import {
+
+#$::dbg->trace('main::','Win32::Daemon::Simple::');
+
 	shift();
 	my $caller_pack = caller;
 	eval {
@@ -91,8 +153,14 @@ sub import {
 				$svcversion = $val;
 			} elsif ($key eq 'info') {
 				$info = $val;
+			} elsif ($key eq 'logging_code') {
+				$logging_code = $val;
 			} elsif ($key eq 'params') {
-				$params = $val;
+				my %params;
+				tie %params, 'Hash::Case::Preserve', $val;
+				$params = \%params;
+			} elsif ($key eq 'run_params') {
+				$run_params = $val;
 			} elsif ($key eq 'param_modify') {
 				$param_modify = {};
 				foreach my $param (keys %$val) { # hashes are case sensitive, the param should be INsensitive!
@@ -107,6 +175,7 @@ sub import {
 		};
 
 		if (! ref $params) { $params = {} };
+		if (! ref $run_params) { $run_params = {} };
 		if (! ref $param_modify) { $param_modify = {} };
 		unless (defined $params->{'LogFile'}) {
 			my $logfile = $Bin . '\\' . $Script;
@@ -116,29 +185,36 @@ sub import {
 		} elsif ($params->{'LogFile'} !~ m{^\w:[\\/]}) {
 			$params->{'LogFile'} = $Bin . '\\' . $params->{'LogFile'};
 		}
+		$params->{'Interval'} = 1 unless (defined $params->{'Interval'});
 
 		if (CMDLINE) {
 			Win32::Console::_SetConsoleTitle( "$svcname $svcversion (in commandline mode)");
 		}
 
-		if (@ARGV) { # we've got some params !
-			print "$svcname $svcversion\n";
+		my $run_with_params = 0;
+		if ($^C != 1 and @ARGV) { # we've got some params !
+			$run_with_params = 1 if (grep {$_ eq '--'} @ARGV);
+			print "$svcname $svcversion\n" unless $run_with_params;
 			my $inst = 0;
 			my $re = join '|', map {quotemeta $_} (keys %$params);
 			my $nore = qr{^[-/]no($re)$}i;
 			my $defre = qr{^[-/]DEFAULT($re)$}i;
 			$re = qr{^[-/]($re)(?:=(.*))?$}si;
 
-			foreach my $opt (@ARGV) {
+			while (my $opt = shift(@ARGV)){
 				if ($opt =~ m{^[-/]install}i) {
 					$info->{'name'} = $svcid;
 					$info->{'display'} = $svcname unless defined $info->{'display'};
 					if (! exists $info->{'path'}) {
-						if ($0 !~ /\.exe$/i) {
-							$info->{'path'} =  $^X;
-							$info->{'parameters'} = "$Bin\\$Script"
-						} else {
+						if ($compiled or $0 =~ /\.exe$/i) {
 							$info->{'path'} = "$Bin\\$Script";
+						} else {
+							$info->{'path'} =  $^X;
+							if (defined $info->{'parameters'}) {
+								$info->{'parameters'} = "$Bin\\$Script $info->{'parameters'}";
+							} else {
+								$info->{'parameters'} = "$Bin\\$Script";
+							}
 						}
 					}
 					Win32::Daemon::DeleteService($svcid);
@@ -168,7 +244,13 @@ sub import {
 						MsgBox "Failed to uninstall: " . Win32::FormatMessage( Win32::Daemon::GetLastError() ) . "\n", MB_ICONERROR, $svcname;
 					}
 					$inst = 1;
-				} elsif ($opt =~ m{^[-/](?:help|\?)}i) {
+				} elsif ($opt =~ m{^[-/]start}i) {
+					StopStartService('start');
+					$inst = 1;
+				} elsif ($opt =~ m{^[-/]stop}i) {
+					StopStartService('stop');
+					$inst = 1;
+				} elsif ($opt =~ m{^[-/](?:help(?:_and_settings)?|\?)$}i) {
 					my $dsc = $params->{'Description'};
 					$dsc =~ s/\n/\n      /g;
 					print <<"*END*";
@@ -179,25 +261,90 @@ $Script -install
 $Script -uninstall
   : uninstalls the service
 
+$Script -start
+  : starts the service
+
+$Script -stop
+  : stops the service
+
 $Script -params
   : displays the effective settings
 
-$Script -PARAM=VALUE
-  : changes the value of an option (you may specify several params at once)
-      LogFile : path to the log file
-      $dsc
-
 $Script -PARAM
   : changes the value of the option to 1
 
 $Script -noPARAM
   : changes the value of the option to 0
+
+$Script -PARAM=VALUE
+  : changes the value of an option (you may specify several params at once)
+
+  The available options:
+      $dsc
+
 *END*
 					if (! FROMCMDLINE) {
-						print "(press ENTER to exit)\n";
-						<STDIN>;
+						if ($opt =~ m{^[-/]help_and_settings$}i) {
+							print <<'*END*';
+Type "continue" and press ENTER to start processing files in commandline mode,
+type "install" and press ENTER to install the service,
+type "uninstall" and press ENTER to uninstall the service,
+type "start" and press ENTER to start the service,
+type "stop" and press ENTER to stop the service,
+type "ParameterName=Value" and press ENTER to change a service parameter
+or press ENTER to exit
+*END*
+							while (my $line = <STDIN>) {
+								chomp($line);
+								last if $line eq '';
+								my ($param, $value) = split /=/, $line, 2;
+								$param = lc $param;
+								if ($param eq 'install') {
+									push @ARGV, '-install';
+									last;
+								} elsif ($param eq 'uninstall') {
+									push @ARGV, '-uninstall';
+									last;
+								} elsif ($param eq 'start') {
+									push @ARGV, '-start';
+									last;
+								} elsif ($param eq 'stop') {
+									push @ARGV, '-stop';
+									last;
+								} elsif ($param eq 'continue') {
+									$run_with_params = 1;
+									last;
+								} elsif (exists $params->{$param}) {
+									$value = 1 unless defined $value;
+								} elsif (substr($param,0,2) eq 'no' and exists $params->{substr($param,2)}) {
+									$param = substr($param,2);
+									$value = 0 unless defined $value;
+								} else {
+									print "Unknown parameter '$param'!\n";
+									next;
+								}
+								if (exists $param_modify->{$param}) {
+									eval {
+										$value = $param_modify->{$param}->($value);
+										SaveParam( $param, $value);
+										$run_params->{$param} = $value;
+										print "    $param: $value\n";
+									};
+									if ($@) {
+										print "    $param: $@\n";
+									}
+								} else {
+									SaveParam( $param, $value);
+									$run_params->{$param} = $value;
+									print "    $param: $value\n";
+								}
+							}
+						} else {
+							print "(Press ENTER to exit)\n";
+							<STDIN>;
+						}
 					}
-					exit();
+#					exit();
 				} elsif ($opt =~ m{^[-/]params}i) {
 					foreach my $param (keys %$params) {
 						next if lc($param) eq 'description';
@@ -216,18 +363,24 @@ $Script -noPARAM
 				} elsif ($opt =~ $re) {
 					my ( $opt, $val) = ( lc($1), $2);
 					$val = 1 unless defined $val;
-					if (exists $param_modify->{lc $opt}) {
+					if (exists $param_modify->{$opt}) {
 						eval {
-							$val = $param_modify->{lc $opt}->($val);
-							print "    $opt: $val\n";
-							SaveParam( $opt, $val);
+							$val = $param_modify->{$opt}->($val);
+							$run_params->{$opt} = $val;
+							unless ($run_with_params) {
+								print "    $opt: $val\n";
+								SaveParam( $opt, $val);
+							}
 						};
 						if ($@) {
 							print "    $opt: $@\n";
 						}
 					} else {
-						print "    $opt: $val\n";
-						SaveParam( $opt, $val);
+						$run_params->{$opt} = $val;
+						unless ($run_with_params) {
+							print "    $opt: $val\n";
+							SaveParam( $opt, $val);
+						}
 					}
 				} elsif ($opt =~ $nore) {
 					my ( $opt) = lc($1);
@@ -235,90 +388,121 @@ $Script -noPARAM
 					if (exists $param_modify->{lc $opt}) {
 						eval {
 							$val = $param_modify->{lc $opt}->($val);
-							print "    $opt: $val\n";
-							SaveParam( $opt, $val);
+							$run_params->{$opt} = $val;
+							unless ($run_with_params) {
+								print "    $opt: $val\n";
+								SaveParam( $opt, $val);
+							}
 						};
 						if ($@) {
 							print "    $opt: $@\n";
 						}
 					} else {
-						print "    $opt: $val\n";
-						SaveParam( $opt, $val);
+						$run_params->{$opt} = $val;
+						unless ($run_with_params) {
+							print "    $opt: $val\n";
+							SaveParam( $opt, $val);
+						}
 					}
 				} elsif ($opt =~ $defre) {
 					my ( $opt) = lc($1);
-					my $val = undef;
-					SaveParam( $opt, $val);
-					print "    $opt: -DEFAULT-VALUE-\n";
+					my $val = $params->{$opt};
+					$run_params->{$opt} = $val;
+					unless ($run_with_params) {
+						print "    $opt: -DEFAULT-VALUE-\n";
+						SaveParam( $opt, $val);
+					}
 				} elsif ($opt =~ m{^[-/]default$}i) {
-					foreach my $param (keys %$params) {
-						SaveParam( $param, $params->{$param});
-						next if lc($param) eq 'description';
-						my $val = $params->{$param};
-						if ($val =~ s/\n/\n        /g) {
-							print "    $param:\n        $val\n";
-						} else {
-							print "    $param: $val\n";
+					if ($run_with_params) {
+						foreach my $param (keys %$params) {
+							$run_params->{lc $param} = $params->{$param};
+						}
+					} else {
+						foreach my $param (keys %$params) {
+							SaveParam( $param, $params->{$param});
+							next if lc($param) eq 'description';
+							my $val = $params->{$param};
+							if ($val =~ s/\n/\n        /g) {
+								print "    $param:\n        $val\n";
+							} else {
+								print "    $param: $val\n";
+							}
 						}
 					}
+				} elsif ($opt eq '--') {
+					die "\$run_with_params was not set !!!" unless $run_with_params; # this should already be set!
+					last;
 				} else {
 					MsgBox "Unknown option '$opt'", MB_ICONEXCLAMATION, $svcname;
 					$inst = 1;
 				}
 			}
 			MsgBox "Changed the options", MB_ICONINFORMATION, $svcname
-				unless $inst;
-			exit; # if we have params
+				unless CMDLINE or $inst or $run_with_params;
+			exit if ($inst or !$run_with_params); # if we have params
 		}
 
 		eval $logging_code; die "$@\n" if $@;
 		$logging_code = '';
 
-		Win32::Daemon::StartService();
+		if (!$^C) {
+			Win32::Daemon::StartService();
 
-		if (CMDLINE) {
-			no warnings qw(redefine);
-			eval "sub Win32::Daemon::State {&SERVICE_START_PENDING}";
-		}
+			if (CMDLINE) {
+				no warnings qw(redefine);
+				eval "sub Win32::Daemon::State {&SERVICE_START_PENDING}";
+			}
 
-		while( &SERVICE_START_PENDING != Win32::Daemon::State() ) {
-			sleep( 1 );
-		}
+			while( SERVICE_START_PENDING != Win32::Daemon::State() ) {
+				sleep( 1 );
+			}
 
-		if (CMDLINE) {
-			no warnings qw(redefine);
-			eval "sub Win32::Daemon::State {&SERVICE_RUNNING}";
-		}
+			if (CMDLINE) {
+				no warnings qw(redefine);
+				eval "sub Win32::Daemon::State {&SERVICE_RUNNING}";
+			}
 
-		LogStart("\n$svcname ver. $svcversion started");
+			LogStart("\n$svcname ver. $svcversion started");
 
-		Win32::Daemon::State( SERVICE_RUNNING );
+			Win32::Daemon::State( SERVICE_RUNNING );
 
-		OpenLog();
-		LogNT("Read params");
-		{
-			local $^W;
-			no strict 'refs';
-			my $val;
+			OpenLog();
+			LogNT("Read params");
+			{
+				local $^W;
+				no strict 'refs';
+				my $val;
+				foreach my $param (keys %$params) {
+					my $sub = uc $param;
+					if (exists $run_params->{lc $param}) {
+						$val = $run_params->{lc $param};
+					} else {
+						$val = ReadParam( $param, $params->{$param});
+					}
+					LogNT("\t$param: $val") unless lc($param) eq 'description';
+					if (! defined $val) {
+						eval "sub $sub () {undef}";
+					} elsif ($val =~ /^\d+(?:\.\d+)?$/) { # if it looks like a number it IS a number
+						$val += 0;
+						eval "sub $sub () {$val}";
+					} else {
+						$val =~ s{(['\\])}{\\$1}g;
+						eval "sub $sub () {'$val'}";
+					}
+					push @EXPORT, $sub;
+				}
+			}
+			LogNT('Running');
+			CloseLog();
+			eval $loop_code; die "$@\n" if $@;
+			$loop_code = '';
+		} else {
 			foreach my $param (keys %$params) {
 				my $sub = uc $param;
-				$val = ReadParam( $param, $params->{$param});
-				LogNT("\t$param: $val") unless lc($param) eq 'description';
-				if ($val =~ /^\d+(?:\.\d+)?$/) { # if it looks like a number it IS a number
-					$val += 0;
-					eval "sub $sub () {$val}";
-				} else {
-					$val =~ s{(['\\])}{\\$1}g;
-					eval "sub $sub () {'$val'}";
-				}
+				eval "sub $sub () {}";
 				push @EXPORT, $sub;
 			}
 		}
-		LogNT('Running');
-		CloseLog();
-		eval $loop_code; die "$@\n" if $@;
-		$loop_code = '';
-
 	};
 	if ($@) {
 		if (CMDLINE) {
@@ -332,7 +516,7 @@ $Script -noPARAM
 		} else {
 			exit(); # don't have a way to report the problem. The person should have tried it in commandline mode first.
 		}
-	} else {
+	} elsif (! $^C) { # if not being compiled
 		SaveParam("ERROR", undef);
 	};
 
@@ -352,7 +536,7 @@ END {
 
 sub ServiceLoop {
 	my $process = shift();
-	my $cnt = int(INTERVAL * 60);
+	my $cnt = 1;
 	my $tick_cnt = 60;
 	my $state;
 	while (1) {
@@ -394,7 +578,7 @@ sub ServiceLoop {
 				}
 			}
 			if (TICK and (--$tick_cnt == 0)) {
-				Log('tick') ;
+				LogNT('tick: '.Now()) ;
 				$tick_cnt = 60;
 			}
 			sleep 1;
@@ -527,7 +711,7 @@ $logging_code = <<'-END--';
 		$logfile = ReadParam('LogFile', $params->{'LogFile'})
 			unless $logfile;
 		open $LOG, ">> $logfile";
-		print $LOG @_, " at ",scalar(localtime()),"\n";
+		print $LOG @_, " at ", Now(),"\n";
 		print STDOUT @_,"\n" if CMDLINE;
 		close $LOG;
 	}
@@ -539,7 +723,7 @@ $logging_code = <<'-END--';
 				unless -e $logfile;
 			open $LOG, ">> $logfile";
 		}
-		print $LOG @_, " at ",scalar(localtime()),"\n";
+		print $LOG @_, " at ", Now(),"\n";
 		$messages .= join '', @_,"\n"
 			if $catchmessages;
 		print STDOUT @_,"\n"
@@ -585,6 +769,9 @@ $logging_code = <<'-END--';
 		$messages = '';
 		return $msg;
 	}
+	sub RedirectLog {
+		$logfile = shift();
+	}
 }
 -END--
 
@@ -596,6 +783,8 @@ __END__
 
 Win32::Daemon::Simple - framework for Windows services
 
+0.2.2
+
 =head1 SYNOPSIS
 
 	use FindBin qw($Bin $Script);
@@ -610,14 +799,13 @@ Win32::Daemon::Simple - framework for Windows services
 			user    =>  '',
 			pwd     =>  '',
 			interactive => 0,
-	#		path    =>  $^X,
-	#		parameters => "$Bin\\$Script",
+	#		parameters => "-- foo bar baz",
 		},
-		Params => {
+		Params => { # the default parameters
 			Tick => 0,
 			Talkative => 0,
 			Interval => 10, # minutes
-			LogFile => "$Bin\\Import",
+			LogFile => "ServiceName.log",
 			# ...
 			Description => <<'*END*',
 	Tick : (0/1) controls whether the service writes a "tick" message to
@@ -629,10 +817,19 @@ Win32::Daemon::Simple - framework for Windows services
 	...
 	*END*
 		},
-		param_modify => {
+		Param_modify => {
 			LogFile => sub {File::Spec->rel2abs($_[0])},
-			Interval => sub {no warnings;my $interval = 0+$_[0]; die "The interval must be a positive number!\n" unless $interval > 0;return $interval},
+			Interval => sub {
+				no warnings;
+				my $interval = 0+$_[0];
+				die "The interval must be a positive number!\n"
+					unless $interval > 0;
+				return $interval
+			},
 			Tick => sub {return ($_[0] ? 1 : 0)},
+		},
+		Run_params => { # parameters for this run of the service
+			#...
 		};
 
 	# initialization
@@ -663,7 +860,155 @@ This module should allow you to create your services in a simple and consistent
 service name and other settings and the actuall processing, the service related stuff and commandline
 parameters are taken care off already.
 
-=head2 EXPORT
+=head2 use Win32::Daemon::Simple
+
+All the service parameters are passed to the module via the use statement. This allows the module to fetch
+the service parameters before your script gets compiled, set the constants according to the parameters and
+to the way the script was started. Thanks to this Perl will be able to inline the constant values and optimize out
+statements that are not needed. Eg:
+
+	print "This will print only if you start the script on cmd line.\n"
+		if CMDLINE;
+
+=head3 Service
+
+The internal system name of the service (for example "w3svc"). The service parameters
+will be stored in the registry in HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\$Service.
+
+=head3 Name
+
+The name of the service as it will be printed into the log file and to the screen when
+installing/uninstalling/modifying the service.
+
+=head3 Version
+
+The version number. This will be printed to the screen and log files and used by PDKcompile to set
+the version info of the EXE generated by PerlApp.
+
+=head3 Info
+
+This is a hash that is with minor changes passed to the Win32::Daemon::CreateService.
+
+=head4 display
+
+The display name of the service. This is the name that will be displayed in the Service Manager.
+Eg. "World Wide Web Publishing Service".
+
+=head4 description
+
+The description displayed alongside the display name in the Service Manager.
+
+=head4 user
+
+=head4 pwd
+
+The username and password that the service will be running under.
+
+=head4 interactive
+
+Whether or not is the service supposed to run interactive (visible to whoever is logged on the server's console).
+
+=head4 path
+
+The path to the script/program to run. This should either be full path to Perl, space and full path to your raw script
+OR a full path to the EXE created by PerlApp or Perl2Exe. This option will be set properly by the module and you
+should never specify it yourself. You should really know what you are doing and what before you do.
+
+=head4 parameters
+
+The "command line" parameters that are to be passed to the service. Please see below for the explanation of
+commandline parameter processing !!!
+
+=head3 Params
+
+This hash specifies the parameters that the service uses and their DEFAULT values.
+When the service is installed these values will be stored in the registry
+(under HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\$Service\Parameters)
+and whenever the service starts the current values will be read and the module will define a constant
+for each of the subkeys.
+
+Some of the parameters are used by the Win32::Daemon::Simple itself, so they should be always present.
+
+=head4 Tick
+
+Controls whether the module prints
+
+	"tick: " . strftime( "%Y/%m/%d %H:%M:%S", localtime()) . "\n"
+
+into the log file once a minute. (So that you could see that the service did not hang, but just doesn't have anything to do.)
+The ticking will be done only if you let the module do the looping (see C<ServiceLoop> below).
+
+If you do not specify this parameter here it will always be OFF.
+
+=head4 Interval
+
+Specifies how often should the module call your callback subroutine (see C<ServiceLoop> below).
+In minutes, though it doesn't have to be a whole number, you can specify interval=0.5.
+The module will not call your callback more often than once a second though!
+
+Not necessary if you do the looping yourself.
+
+=head4 LogFile
+
+The path to the log file. You should include this parameter so that the user will be able to change the path where
+the logging information is written. Currently it's not possible to turn the loggin off except by overwriting
+the C<Logging_code>.
+
+If you do not specify this parameter or use C<undef> then the log file will be created in the same directory as the script
+and named ScriptName.log.
+
+=head4 Description
+
+This value of this parameter is included in the help printed when the script is executed with -help parameter.
+It should describe the various parameters that you can set for the service.
+
+The values of these options are available as TICK, INTERVAL, LOGFILE and DESCRIPTION constants.
+
+=head3 Param_modify
+
+Here you may specify what functions to call when the user tries to update a service parameter.
+The function may modify or reject the new value. If you want to reject a value die("with the message\n"),
+otherwise return the value you want to be stored in the registry and used by the service.
+
+		Param_modify => {
+			LogFile => sub {File::Spec->rel2abs($_[0])},
+			Interval => sub {
+				no warnings;
+				my $interval = 0+$_[0];
+				die "The interval must be a positive number!\n"
+					unless $interval > 0;
+					return $interval
+				},
+			Tick => sub {return ($_[0] ? 1 : 0)},
+		},
+
+=head3 Logging_code
+
+(ADVANCED) This option allows you to overwrite the functions that will be used for logging.
+You can log into the EvenLog or whereever you like.
+
+		Logging_code => <<'*END*',
+	sub LogStart {};	# called once when the service starts
+	sub Log {};		# called many times. Appends a timestamp.
+	sub LogNT {};		# called many times. Doesn't append a timestamp.
+	sub OpenLog {};	# called once, just before printing the params
+	sub CloseLog {};	# called once, just after printing the params
+	sub CatchMessages {}; # not caled by Win32::Daemon::Simple
+	sub GetMessages {}; # not caled by Win32::Daemon::Simple
+	*END*
+
+See below for more information about the functions.
+
+=head3 Run_params
+
+(ADVANCED) Here you can overwrite the service parameters. The values specified here take precedence over
+the values stored in the registry or specified in Params=> hash.
+
+	Run_params => {
+		LogFile => (condition ? "$Bin\\Foo.log" : "$Bin\\Bar.log"),
+	}
+
+=head2 Exported functions
 
 =head3 ServiceLoop
 
@@ -721,7 +1066,7 @@ If $StopProc is:
 	Pause()
 	Pause($UnPauseProc, $StopProc)
 
-If the DoEvents() returned SERVICE_STOP_PENDING you should do whatever you need
+If the DoEvents() returned SERVICE_PAUSE_PENDING you should do whatever you need
 to get the service to a pausable state (close open database connections etc.) and
 call this procedure. The meanings of the parameters is the same as for DoEvents().
 
@@ -777,6 +1122,12 @@ as a default) and defines a constant named C<uc($parametername)>.
 
 =head2 Service parameters
 
+The parameters passed to a script using this module will be processed by the module!
+If you want to pass some paramters to the script itself use -- as a parameter.
+If you do then the parameters before the -- will be processed by the module and the ones behind
+will be passed to the script. If you do not use the -- but do call the program with some parameters
+then the parameters will be processed and your program will end!
+
 The service created using this module will accept the following commandline parameters:
 
 =head3 -install
@@ -798,6 +1149,14 @@ close the Services window and/or the regedit and try again!
 
 Uninstalls the service.
 
+=head3 -start
+
+Starts the service.
+
+=head3 -stop
+
+Stops the service.
+
 =head3 -params
 
 Prints the actual values of all the parameters of the service.
@@ -817,14 +1176,14 @@ Sets the value of PARAM to 1. The parameter names are case insensitive.
 
 =head3 -noPARAM
 
-Sets the value of PARAM to 1. The parameter names are case insensitive.
+Sets the value of PARAM to 0. The parameter names are case insensitive.
 
 =head3 -PARAM=value
 
 Sets the value of PARAM to value. The parameter names are case insensitive.
 
 You may validate and/or modify the value with a handler specified in the
-param_modify=>{} option. If the handler die()s the value will NOT be changed
+Param_modify=>{} option. If the handler die()s the value will NOT be changed
 and the error message will be printed to the screen.
 
 =head3 -defaultPARAM
@@ -832,25 +1191,72 @@ and the error message will be printed to the screen.
 Deletes the parameter from registry, therefore the default value of that parameter
 will be used each time the service starts.
 
-=head3 Comments
+=head3 --
+
+Stop processing parameters, run the script and leave the rest of @ARGV intact.
+The -install, -uninstall, -stop, -start, -help and -params parameters cannot be used
+before the --.
+
+If the service parameters contain -- then all the -param, -noparam, -param=value,
+-defaultparam and -default only affect the current run and are not written into the registry.
+
+
+=head3 Examples
+
+	script.pl -install
+
+Installs the script.pl as a service with the default parameters.
+
+	script.pl -uninstall
+
+Uninstalls the service.
+
+	script.pl -tick -interval=10
+
+Changes the options in the registry. When the service starts next time it will tick and the callbacl will be called
+each 10 minutes.
+
+	script.pl -notick -interval=5 --
+
+Start the service without ticking and with the interval of 5 minutes. Do not make any changes to the registry.
+
+	script.pl -interval=60 -start
+
+Set the interval to 60 minutes in the registry, start the service (via the service manager) and exit.
+
+	script.pl -- foo fae fou
+
+Start the service and set @ARGV = qw(foo fae fou).
+
+=head2 Comments
 
 The scripts using this module are sensitive to the way they were started.
 
-If you start them with a parameter they process that parameter as explained abovet.
+If you start them with a parameter they process that parameter as explained above.
 Then if you started them from the Run dialog or by doubleclicking they print
 (press ENTER to continue) and wait for the user to press enter, if you started them from
 the command prompt they exit immediately
 
-If they are started without parameters by the Service Manager they register with
-the Manager and start your code, if they are started without parameters from command prompt
+If they are started without parameters or with -- by the Service Manager they register with
+the Manager and start your code passing it whatever parameters you specified after the --,
+if they are started without parameters from command prompt
 they start working in a command line mode (all info is printed to the screen as well as to the log file)
 and if they are started by doubleclicking on the script they show the -help screen.
 
+=head2 To do
+
+-install=name
+	A way to override the service name set by the script. I  will have to append
+	-s_v_c_n_a_m_e=name to the service parameters!
+	Needed for ability to run several instances of a service.
+
 =head1 AUTHOR
 
  Jenda@Krynicky.cz
  http://Jenda.Krynicky.cz
 
+ With comments and suggestions by extern.Lars.Oeschey@audi.de
+
 =head1 SEE ALSO
 
 L<Win32::Daemon>.
